from django.contrib.sites.shortcuts import get_current_site
from django.http import Http404, HttpResponse
from django.shortcuts import get_object_or_404
from django.views.decorators.csrf import csrf_protect
from django.views.generic.list import ListView

from simpel_hookup import core as hookup

from simpel_pages.models import Category, Tag
from simpel_pages.settings import pages_settings

from .helpers import log_visitor
from .models import RootPage


@csrf_protect
def serve_page(request, path):
    # we need a valid Site object corresponding to this request in order to proceed
    site = get_current_site(request)
    if not site:
        raise Http404

    path_components = [component for component in path.split("/") if component]
    site_root = RootPage.objects.get(site=site)
    page, args, kwargs = site_root.page.specific.route(request, path_components)

    for fn in hookup.get_hooks("PAGES_BEFORE_SERVE"):
        result = fn(page, request, args, kwargs)
        if isinstance(result, HttpResponse):
            return result
    log_visitor(request, content=page)
    if request.user.is_authenticated:
        page.readers.add(request.user)
    return page.serve(request, *args, **kwargs)


@csrf_protect
def serve_category(request, slug):
    category = get_object_or_404(Category, slug=slug)
    site = get_current_site(request)
    site_root = RootPage.objects.get(site=site)
    real_model = request.GET.get("type", None)
    filters = dict()
    if real_model is not None:
        filters["real_model"] = real_model
    filters["category__slug__in"] = [category.slug]
    queryset = site_root.page.get_descendants().filter(**filters).live()
    log_visitor(request, content=category)
    return ListView.as_view(
        queryset=queryset,
        paginate_by=pages_settings.ITEM_PER_PAGE,
        template_name="simpel_pages/category.html",
        extra_context={"object": category},
    )(request)


@csrf_protect
def serve_tag(request, slug):
    tag = get_object_or_404(Tag, slug=slug)
    site = get_current_site(request)
    site_root = RootPage.objects.get(site=site)
    real_model = request.GET.get("type", None)
    filters = dict()
    if real_model is not None:
        filters["real_model"] = real_model
    filters["tags__slug__in"] = [tag.slug]
    queryset = site_root.page.get_descendants().filter(**filters).live()
    log_visitor(request, content=tag)
    return ListView.as_view(
        queryset=queryset,
        paginate_by=pages_settings.ITEM_PER_PAGE,
        template_name="simpel_pages/tag.html",
        extra_context={"object": tag},
    )(request)


def serve_tags(request):
    return ListView.as_view(
        queryset=Tag.objects.all(),
        paginate_by=pages_settings.CATEGORY_PER_PAGE,
        template_name="simpel_pages/tag_list.html",
    )(request)


def serve_categories(request):
    return ListView.as_view(
        queryset=Category.objects.all(),
        paginate_by=pages_settings.TAG_PER_PAGE,
        template_name="simpel_pages/category_list.html",
    )(request)



from copy import copy

from django.contrib import admin
from django.core.exceptions import ImproperlyConfigured
from django.utils import timezone
from django.utils.html import format_html
from django.utils.text import Truncator
from django.utils.translation import gettext_lazy as _

from mptt.admin import DraggableMPTTAdmin
from polymorphic.admin import PolymorphicInlineSupportMixin
from simpel_hookup import core as hookup

from simpel_pages.admin.blocks import BlockInline

from ..models import Category, Page, RootPage, Tag, Visitor


@admin.register(Category)
class CategoryAdmin(DraggableMPTTAdmin):
    search_fields = ["name", "slug"]
    list_display = list(DraggableMPTTAdmin.list_display) + ["name", "slug", "description"]


@admin.register(Tag)
class TagAdmin(admin.ModelAdmin):
    search_fields = ["name", "description"]
    list_display = ["name", "slug", "description"]


@admin.register(Page)
class PageAdmin(PolymorphicInlineSupportMixin, DraggableMPTTAdmin):
    list_filter = ("index", "live", "allow_comments", "registration_required")
    list_display = [
        "tree_actions",
        "indented_title",
        "page_type",
        "visitor",
        "bookmark",
        "reader",
        "live",
    ]
    search_fields = ("url_path", "title")
    seo_settings = (
        _("SEO Settings"),
        {
            "fields": (
                "slug",
                "seo_title",
                "seo_description",
                "data",
                "template",
                "allow_comments",
                "registration_required",
                "index",
                "per_page",
                "live",
            ),
            "classes": ["collapse"],
        },
    )
    page_fields = [
        "parent",
        "title",
        "tags",
        "category",
        "thumbnail",
        "content",
    ]
    fieldsets = (
        (None, {"fields": page_fields}),
        seo_settings,
    )
    inlines = [BlockInline]

    def get_inlines(self, request, obj):
        funcs = hookup.get_hooks("SIMPEL_PAGES_PAGE_INLINES")
        inlines = copy(self.inlines)
        for func in funcs:
            inline_class = func()
            if not issubclass(
                inline_class,
                admin.options.InlineModelAdmin,
            ):
                raise ImproperlyConfigured(_("%s must subclass InlineModelAdmin"))
            inlines.append(inline_class)
        return super().get_inlines(request, obj)

    def get_page_types(self):
        funcs = hookup.get_hooks("SIMPEL_PAGES_PAGE_TYPES")
        types = []
        for func in funcs:
            page_model = func()
            if not issubclass(page_model, Page):
                raise ImproperlyConfigured(_("%s must subclass simpel_pages.Page"))
            types.append(page_model)
        return types

    def bookmark(self, obj):
        return obj.bookmarks.count()

    def reader(self, obj):
        return obj.readers.count()

    def visitor(self, obj):
        return obj.visitor_count

    def indented_title(self, item):
        item_text = Truncator(item).chars(60)
        return format_html(
            '<div style="text-indent:{}px">{}</div>',
            item._mpttfield("level") * self.mptt_level_indent,
            item_text,
        )

    def save_model(self, request, obj, form, change):
        if obj.owner is None:
            obj.owner = request.user
        obj.updated_at = timezone.now()
        return super().save_model(request, obj, form, change)


@admin.register(RootPage)
class SiteRootAdmin(admin.ModelAdmin):
    list_display = ["site", "page"]
    pass


@admin.register(Visitor)
class VisitorAdmin(admin.ModelAdmin):
    list_display = ["created_at", "url", "ip", "referrer"]



from polymorphic.admin import GenericStackedPolymorphicInline

from simpel_pages.models.blocks import RichTextBlock

from ..models import Block, ImageBlock


class BlockInline(GenericStackedPolymorphicInline):
    model = Block
    ct_field = "object_type"

    def get_child_models(self):
        """
        Register child model using defaults from settings
        """
        # Get deliverable child models map from hooks
        child_models = []
        if len(child_models) == 0:
            child_models = [RichTextBlock, ImageBlock]
        return child_models

    def get_child_inline_classes(self):
        child_models = self.get_child_models()
        child_inlines = []
        for child in child_models:
            class_name = child.__class__.__name__
            props = {"model": child, "ct_field": "object_type"}
            parent_class = (GenericStackedPolymorphicInline.Child,)
            inline_class = type("%sInline" % class_name, parent_class, props)
            child_inlines.append(inline_class)
        return child_inlines

    def get_child_inline_instances(self):
        instances = []
        for ChildInlineType in self.get_child_inline_classes():
            instances.append(ChildInlineType(parent_inline=self))
        return instances



import hashlib

from django.core.cache import cache

from simpel_pages.utils import get_ip

from .models import Visitor
from .settings import pages_settings


def log_visitor(request, status_code=None, content=None):
    ip = get_ip(request)
    url = request.build_absolute_uri()
    cache_key = "Visitor:%s" % hashlib.sha1(f"{ip}-{url}".encode("utf-8")).hexdigest()
    timeout = pages_settings.VISITOR_CACHE_TIMEOUT
    last_visit = cache.get(cache_key)
    if not last_visit:
        if pages_settings.VISITOR_SAVE:
            referrer = request.META.get("HTTP_REFERER")
            data = {
                "ip": ip,
                "url": url,
                "path": request.get_full_path(),
                "referrer": referrer,
                "content": content,
            }
            visitor = Visitor(**data)
            visitor.save()
        if getattr(content, "visitor_count", None) is not None:
            content.visitor_count += 1
            content.save()
        cache.set(cache_key, 1, timeout)



from django import template
from django.template import loader

from ..models import Category, Page, Tag

register = template.Library()


@register.simple_tag(takes_context=True)
def popular_pages(context, title, obj=None, number=5, template_name=None):
    if obj is not None:
        qs = obj.childs.all()
    else:
        qs = Page.objects.all()
    qs = qs.filter(index=False).live().order_by("-visitor_count")[:number]
    context = {"title": title, "pages": qs}
    if template_name is None:
        template_name = "simpel_pages/widgets/page_list.html"
    template = loader.get_template(template_name)
    return template.render(context)


@register.simple_tag(takes_context=True)
def recent_pages(context, title, obj=None, number=5, template_name=None):
    if obj is not None:
        qs = obj.childs.all()
    else:
        qs = Page.objects.all()
    qs = qs.filter(index=False).live().order_by("-created_at")[:number]
    context = {"title": title, "pages": qs}
    if template_name is None:
        template_name = "simpel_pages/widgets/page_list.html"
    template = loader.get_template(template_name)
    return template.render(context)


@register.simple_tag(takes_context=True)
def related_pages_by_category(context, title, obj, number=5, template_name=None):
    qs = obj.get_related_by_category(number)
    context = {"title": title, "pages": qs}
    if template_name is None:
        template_name = "simpel_pages/widgets/page_list.html"
    template = loader.get_template(template_name)
    return template.render(context)


@register.simple_tag(takes_context=True)
def related_pages_by_tags(context, title, obj, number=5, template_name=None):
    qs = obj.get_related_by_tags(number)
    context = {"title": title, "pages": qs}
    if template_name is None:
        template_name = "simpel_pages/widgets/page_list.html"
    template = loader.get_template(template_name)
    return template.render(context)


@register.simple_tag(takes_context=True)
def related_pages(context, title, obj, number=5, by=None, template_name=None):
    if by == "tags":
        return related_pages_by_tags(context, title, obj, number, template_name)
    else:
        return related_pages_by_category(context, title, obj, number, template_name)


@register.simple_tag(takes_context=True)
def category_list(context, title, number=5, template_name=None):
    qs = Category.objects.all()
    context = {"title": title, "categories": qs[:number]}
    if template_name is None:
        template_name = "simpel_pages/widgets/category_list.html"
    template = loader.get_template(template_name)
    return template.render(context)


@register.simple_tag(takes_context=True)
def tags_list(context, title, number=5, template_name=None):
    qs = Tag.objects.popular()
    context = {"title": title, "tags": qs[:number]}
    if template_name is None:
        template_name = "simpel_pages/widgets/tags_list.html"
    template = loader.get_template(template_name)
    return template.render(context)



from django.db import models
from django.utils import timezone

from mptt.managers import TreeManager, TreeQuerySet


class TagManager(models.Manager):
    def get_queryset(self):
        qs = super().get_queryset().prefetch_related("tagged_pages")
        count = models.Count("tagged_pages", distinct=True)
        return qs.prefetch_related("tagged_pages").annotate(tagged_count=count).order_by("name")

    def popular(self):
        return self.get_queryset().order_by("-tagged_count")


class CategoryManager(TreeManager):
    def get_queryset(self, *args, **kwargs):
        qs = super().get_queryset(*args, **kwargs)
        return qs.select_related("parent")


class PageQueryset(TreeQuerySet):
    def as_manager(cls):
        # Address the circular dependency between `Queryset` and `Manager`.
        from simpel_pages.managers import PageManager

        manager = PageManager.from_queryset(cls)()
        manager._built_with_as_manager = True
        return manager

    as_manager.queryset_only = True
    as_manager = classmethod(as_manager)

    def live(self):
        return self.filter(live=True)

    def _get_real_model_name(self, model):
        if issubclass(model, models.Model):
            opts = model._meta
            model_name = "%s.%s" % (opts.app_label, opts.model_name)
        if isinstance(model, (str,)):
            model_name = model
        return model_name

    def instance_of(self, model):
        if isinstance(model, (list, set, tuple)):
            models = [m for m in self._get_real_model_name(model)]
        else:
            models = [self._get_real_model_name(model)]
        return self.filter(real_model__in=models)


class PageManager(models.Manager.from_queryset(PageQueryset), TreeManager):
    def get_queryset(self, *args, **kwargs):
        qs = super().get_queryset(*args, **kwargs)
        return qs.select_related("parent")

    def get_related_by_category(self, obj, tree=True, full=False):
        qs = self.get_queryset()
        cat = obj.category
        if cat is None:
            return qs.none()
        if tree:
            cats = [cat]
            if full:
                cats += list(cat.get_descendants())
            else:
                cats += list(cat.children.all())
        else:
            cats = [cat]
        slugs = [c.slug for c in cats]
        qs = self.get_queryset()
        return qs.filter(category__slug__in=slugs).exclude(pk=obj.id).live()

    def get_related_by_tags(self, obj):
        tags = obj.tags.all()
        slugs = [tag.slug for tag in tags]
        qs = self.get_queryset()
        return qs.filter(tags__slug__in=slugs).distinct().exclude(pk=obj.id).live()

    def get_most_viewed(self, from_date=None, to_date=None):
        qs = self.get_queryset()
        if to_date is None:
            to_date = timezone.now()
        return qs.filter(created_at__range=[from_date, to_date])



from django.urls import re_path

from . import views
from .settings import pages_settings

if pages_settings.APPEND_SLASH:
    # If APPEND_SLASH is True (the default value), we match a
    # (possibly empty) list of path segments ending in slashes.
    # CommonMiddleware will redirect requests without a trailing slash to
    # a URL with a trailing slash
    serve_pattern = r"^((?:[\w\-]+/)*)$"
else:
    # If WAGTAIL_APPEND_SLASH is False, allow Wagtail to serve pages on URLs
    # with and without trailing slashes
    serve_pattern = r"^([\w\-/]*)$"

urlpatterns = [
    re_path(r"^tag/$", views.serve_tags, name="serve_tag"),
    re_path(r"^tag/(?P<slug>[-\w]+)/$", views.serve_tag, name="serve_tag"),
    re_path(r"^category/$", views.serve_categories, name="serve_category"),
    re_path(r"^category/(?P<slug>[-\w]+)/$", views.serve_category, name="serve_category"),
    re_path(serve_pattern, views.serve_page, name="serve_pages"),
]



from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import gettext_lazy as _

from ckeditor.fields import RichTextField
from filer.fields.image import FilerImageField
from polymorphic.models import PolymorphicModel

# Create your models here.


class BaseBlock(PolymorphicModel):
    name = models.SlugField(
        _("name"),
        null=True,
        blank=True,
    )
    group = models.SlugField(
        _("group"),
        null=True,
        blank=True,
    )
    position = models.IntegerField(
        default=0,
        verbose_name=_("position"),
        help_text=_("Used for ordering block."),
    )

    class Meta:
        abstract = True

    def __str__(self):
        return "%s (%s)" % (self.__class__._meta.verbose_name, self.name)


class Block(BaseBlock):
    object_type = models.ForeignKey(
        ContentType,
        related_name="blocks",
        null=True,
        blank=True,
        verbose_name="object type",
        on_delete=models.CASCADE,
    )
    object_id = models.IntegerField(
        null=True,
        blank=True,
        verbose_name=_("object"),
    )
    object = GenericForeignKey(
        "object_type",
        "object_id",
    )

    class Meta:
        verbose_name = _("Block")
        verbose_name_plural = _("Blocks")


class RichTextBlock(Block):
    text = RichTextField()

    class Meta:
        verbose_name = _("Text Block")
        verbose_name_plural = _("Text Blocks")


class ImageBlock(Block):
    caption = models.CharField(
        max_length=200,
        verbose_name=_("Caption"),
    )
    thumb_height = models.IntegerField(
        default=100,
        verbose_name=_("Thumbnail Height"),
    )
    thumb_width = models.IntegerField(
        default=100,
        verbose_name=_("Thumbnail Width"),
    )
    image = FilerImageField(
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
    )

    class Meta:
        verbose_name = _("Image Block")
        verbose_name_plural = _("Image Blocks")
